package pp.battleship.client;


import com.jme3.app.Application;
import com.jme3.app.state.AbstractAppState;
import com.jme3.app.state.AppStateManager;
import com.jme3.asset.AssetLoadException;
import com.jme3.asset.AssetNotFoundException;
import com.jme3.audio.AudioData;
import com.jme3.audio.AudioNode;


import java.lang.System.Logger;
import java.lang.System.Logger.Level;
import java.util.prefs.Preferences;

import static pp.util.PreferencesUtils.getPreferences;

/**
 * An application state that plays music.
 */
public class GameMusic extends AbstractAppState {
    private static final Logger LOGGER = System.getLogger(GameMusic.class.getName());
    private static final Preferences PREFERENCES = getPreferences(GameMusic.class);
    private static final String ENABLED_PREF = "enabled"; //NON-NLS
    private static final String VOLUME_PREF = "volume"; //NON-NLS

    private AudioNode battleMusic;
    private AudioNode menuMusicModern;
    private AudioNode menuMusicTraditional;

    /**
     * Checks if sound is enabled in the preferences.
     *
     * @return {@code true} if sound is enabled, {@code false} otherwise.
     */
    public static boolean enabledInPreferences() {
        return PREFERENCES.getBoolean(ENABLED_PREF, true);
    }
    /**
     * Checks enabled volume in preferences.
     *
     * @return {@code float} if a volumePreference is set, the volume is set to the Value in  PREFERENCES,
     *         {@code 0.25f} if no volumePreference is set in  PREFERENCES, the Volume is set to a default of 0.25f
     *
     */

    public static float volumePreference() {
        return PREFERENCES.getFloat(VOLUME_PREF, 0.25f);
    }

    /**
     * Toggles the game sound on or off.
     */
    public void toggleSound() {
        setEnabled(!isEnabled());
    }

    /**
     * Sets the enabled state of this AppState.
     * Overrides {@link com.jme3.app.state.AbstractAppState#setEnabled(boolean)}
     *
     * @param enabled {@code true} to enable the AppState, {@code false} to disable it.
     */
    @Override
    public void setEnabled(boolean enabled) {
        if (isEnabled() == enabled) return;
         else if (!isEnabled() && enabled) {
             if (menuMusicModern != null) menuMusicModern.play();
        } else if (isEnabled() && !enabled) {
             if (menuMusicModern != null) menuMusicModern.stop();
        }

        super.setEnabled(enabled);
        LOGGER.log(Level.INFO, "Music enabled: {0}", enabled); //NON-NLS
        PREFERENCES.putBoolean(ENABLED_PREF, enabled);
    }

    /**
     * Initializes the music.
     * Overrides {@link AbstractAppState#initialize(AppStateManager, Application)}
     *
     * @param stateManager The state manager
     * @param app          The application
     */
    @Override
    public void initialize(AppStateManager stateManager, Application app) {
        super.initialize(stateManager, app);
        menuMusicModern =loadSound(app, "Sound/BackgroundMusic/menu-music-modern.ogg");
        setVolume(volumePreference());
        menuMusicModern.setLooping(true);
        if (isEnabled() && menuMusicModern != null) {
            menuMusicModern.play();
        }

    }

    /**
     * Loads the music from the specified file.
     *
     * @param app  The application
     * @param name The name of the sound file.
     * @return The loaded AudioNode.
     */
    private AudioNode loadSound(Application app, String name) {
        try {
            final AudioNode sound = new AudioNode(app.getAssetManager(), name, AudioData.DataType.Buffer);
            sound.setLooping(false);
            sound.setPositional(false);
            return sound;
        }
        catch (AssetLoadException | AssetNotFoundException ex) {
            LOGGER.log(Level.ERROR, ex.getMessage(), ex);
        }
        return null;
    }

    /**
     * Sets the vol param to the level set in PREFERENCES
     *
     * @param vol  Volume level of the music as indicated by the Volume control Slider
     *
     */
    public void setVolume(float vol){
        menuMusicModern.setVolume(vol);
        PREFERENCES.putFloat(VOLUME_PREF, vol);
    }

}




